@@ -53,11 +53,11 @@ def mTimeVersion(init_file):
     for root, _, files in os.walk(cwd):
         for f in files:
             m = max(os.path.getmtime(os.path.join(root, f)), m)
-    d = datetime.datetime.utcfromtimestamp(m)
+    d = datetime.datetime.fromtimestamp(m, datetime.timezone.utc)
     return d.strftime("%Y.%m.%d")
 
 
-def getVersionFromArchiveId(git_archive_id='1693687476  (tag: v3.9.2, 3.9.x)'):
+def getVersionFromArchiveId(git_archive_id='1720812614  (tag: v3.11.6, 3.11.x)'):
     """ Extract the tag if a source is from git archive.
 
         When source is exported via `git archive`, the git_archive_id init value is modified
@@ -80,7 +80,7 @@ def getVersionFromArchiveId(git_archive_id='1693687476  (tag: v3.9.2, 3.9.x)'):
 
         # archived revision is not tagged, use the commit date
         tstamp = git_archive_id.strip().split()[0]
-        d = datetime.datetime.utcfromtimestamp(int(tstamp))
+        d = datetime.datetime.fromtimestamp(int(tstamp), datetime.timezone.utc)
         return d.strftime('%Y.%m.%d')
     return None
 
