@@ -13,74 +13,159 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import os
 import re
 import stat
 from urllib.parse import quote as urlquote
 
 from twisted.internet import defer
-from twisted.internet import utils
 from twisted.python import log
 
 from buildbot import config
 from buildbot.changes import base
 from buildbot.util import bytes2unicode
+from buildbot.util import giturlparse
 from buildbot.util import private_tempdir
+from buildbot.util import runprocess
 from buildbot.util.git import GitMixin
+from buildbot.util.git import ensureSshKeyNewline
 from buildbot.util.git import getSshKnownHostsContents
 from buildbot.util.misc import writeLocalFile
 from buildbot.util.state import StateMixin
+from buildbot.util.twisted import async_to_deferred
+from buildbot.warnings import warn_deprecated
 
 
 class GitError(Exception):
-
     """Raised when git exits with code 128."""
 
 
-class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
-
+class GitPoller(base.ReconfigurablePollingChangeSource, StateMixin, GitMixin):
     """This source will poll a remote git repo for changes and submit
     them to the change master."""
 
-    compare_attrs = ("repourl", "branches", "workdir",
-                     "pollInterval", "gitbin", "usetimestamps",
-                     "category", "project", "pollAtLaunch",
-                     "buildPushesWithNoCommits", "sshPrivateKey", "sshHostKey",
-                     "sshKnownHosts")
+    compare_attrs = (
+        "repourl",
+        "branches",
+        "workdir",
+        "pollInterval",
+        "gitbin",
+        "usetimestamps",
+        "category",
+        "project",
+        "pollAtLaunch",
+        "buildPushesWithNoCommits",
+        "sshPrivateKey",
+        "sshHostKey",
+        "sshKnownHosts",
+        "pollRandomDelayMin",
+        "pollRandomDelayMax",
+    )
 
     secrets = ("sshPrivateKey", "sshHostKey", "sshKnownHosts")
 
-    def __init__(self, repourl, branches=None, branch=None,
-                 workdir=None, pollInterval=10 * 60,
-                 gitbin='git', usetimestamps=True,
-                 category=None, project=None,
-                 pollinterval=-2, fetch_refspec=None,
-                 encoding='utf-8', name=None, pollAtLaunch=False,
-                 buildPushesWithNoCommits=False, only_tags=False,
-                 sshPrivateKey=None, sshHostKey=None, sshKnownHosts=None):
+    def __init__(self, repourl, **kwargs):
+        self.lastRev: dict[str, str] | None = None
+        name = kwargs.get("name", None)
+        if name is None:
+            kwargs["name"] = repourl
+        super().__init__(repourl, **kwargs)
 
+    def checkConfig(
+        self,
+        repourl,
+        branches=None,
+        branch=None,
+        workdir=None,
+        pollInterval=10 * 60,
+        gitbin="git",
+        usetimestamps=True,
+        category=None,
+        project=None,
+        pollinterval=-2,
+        fetch_refspec=None,
+        encoding="utf-8",
+        name=None,
+        pollAtLaunch=False,
+        buildPushesWithNoCommits=False,
+        only_tags=False,
+        sshPrivateKey=None,
+        sshHostKey=None,
+        sshKnownHosts=None,
+        pollRandomDelayMin=0,
+        pollRandomDelayMax=0,
+    ):
         # for backward compatibility; the parameter used to be spelled with 'i'
         if pollinterval != -2:
             pollInterval = pollinterval
-
-        if name is None:
-            name = repourl
-
-        super().__init__(name=name,
-                         pollInterval=pollInterval,
-                         pollAtLaunch=pollAtLaunch,
-                         sshPrivateKey=sshPrivateKey,
-                         sshHostKey=sshHostKey,
-                         sshKnownHosts=sshKnownHosts)
-
-        if project is None:
-            project = ''
+            warn_deprecated('3.11.3', 'pollinterval has been deprecated: please use pollInterval')
 
         if only_tags and (branch or branches):
             config.error("GitPoller: can't specify only_tags and branch/branches")
         if branch and branches:
             config.error("GitPoller: can't specify both branch and branches")
-        elif branch:
+
+        self.sshPrivateKey = sshPrivateKey
+        self.sshHostKey = sshHostKey
+        self.sshKnownHosts = sshKnownHosts
+        self.setupGit(logname='GitPoller')  # check the configuration
+
+        if fetch_refspec is not None:
+            config.error(
+                "GitPoller: fetch_refspec is no longer supported. "
+                "Instead, only the given branches are downloaded."
+            )
+
+        if name is None:
+            name = repourl
+
+        super().checkConfig(
+            name=name,
+            pollInterval=pollInterval,
+            pollAtLaunch=pollAtLaunch,
+            pollRandomDelayMin=pollRandomDelayMin,
+            pollRandomDelayMax=pollRandomDelayMax,
+        )
+
+    @defer.inlineCallbacks
+    def reconfigService(
+        self,
+        repourl,
+        branches=None,
+        branch=None,
+        workdir=None,
+        pollInterval=10 * 60,
+        gitbin="git",
+        usetimestamps=True,
+        category=None,
+        project=None,
+        pollinterval=-2,
+        fetch_refspec=None,
+        encoding="utf-8",
+        name=None,
+        pollAtLaunch=False,
+        buildPushesWithNoCommits=False,
+        only_tags=False,
+        sshPrivateKey=None,
+        sshHostKey=None,
+        sshKnownHosts=None,
+        pollRandomDelayMin=0,
+        pollRandomDelayMax=0,
+    ):
+        # for backward compatibility; the parameter used to be spelled with 'i'
+        if pollinterval != -2:
+            pollInterval = pollinterval
+            warn_deprecated('3.11.3', 'pollinterval has been deprecated: please use pollInterval')
+
+        if name is None:
+            name = repourl
+
+        if project is None:
+            project = ''
+
+        if branch:
             branches = [branch]
         elif not branches:
             if only_tags:
@@ -95,23 +180,34 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         self.gitbin = gitbin
         self.workdir = workdir
         self.usetimestamps = usetimestamps
-        self.category = category if callable(
-            category) else bytes2unicode(category, encoding=self.encoding)
+        self.category = (
+            category if callable(category) else bytes2unicode(category, encoding=self.encoding)
+        )
         self.project = bytes2unicode(project, encoding=self.encoding)
         self.changeCount = 0
-        self.lastRev = {}
+        self.lastRev = None
         self.sshPrivateKey = sshPrivateKey
         self.sshHostKey = sshHostKey
         self.sshKnownHosts = sshKnownHosts
         self.setupGit(logname='GitPoller')
 
-        if fetch_refspec is not None:
-            config.error("GitPoller: fetch_refspec is no longer supported. "
-                         "Instead, only the given branches are downloaded.")
-
         if self.workdir is None:
             self.workdir = 'gitpoller-work'
 
+        # make our workdir absolute, relative to the master's basedir
+
+        if not os.path.isabs(self.workdir):
+            self.workdir = os.path.join(self.master.basedir, self.workdir)
+            log.msg(f"gitpoller: using workdir '{self.workdir}'")
+
+        yield super().reconfigService(
+            name=name,
+            pollInterval=pollInterval,
+            pollAtLaunch=pollAtLaunch,
+            pollRandomDelayMin=pollRandomDelayMin,
+            pollRandomDelayMax=pollRandomDelayMax,
+        )
+
     @defer.inlineCallbacks
     def _checkGitFeatures(self):
         stdout = yield self._dovccmd('--version', [])
@@ -120,27 +216,21 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         if not self.gitInstalled:
             raise EnvironmentError('Git is not installed')
 
-        if (self.sshPrivateKey is not None and
-                not self.supportsSshPrivateKeyAsEnvOption):
+        if self.sshPrivateKey is not None and not self.supportsSshPrivateKeyAsEnvOption:
             raise EnvironmentError('SSH private keys require Git 2.3.0 or newer')
 
-    @defer.inlineCallbacks
     def activate(self):
-        # make our workdir absolute, relative to the master's basedir
-        if not os.path.isabs(self.workdir):
-            self.workdir = os.path.join(self.master.basedir, self.workdir)
-            log.msg("gitpoller: using workdir '{}'".format(self.workdir))
-
         try:
-            self.lastRev = yield self.getState('lastRev', {})
+            self.lastRev = None
 
             super().activate()
         except Exception as e:
             log.err(e, 'while initializing GitPoller repository')
 
     def describe(self):
-        str = ('GitPoller watching the remote git repository ' +
-               bytes2unicode(self.repourl, self.encoding))
+        str = 'GitPoller watching the remote git repository ' + bytes2unicode(
+            self.repourl, self.encoding
+        )
 
         if self.branches:
             if self.branches is True:
@@ -153,24 +243,28 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
 
         return str
 
-    def _getBranches(self):
-        d = self._dovccmd('ls-remote', ['--refs', self.repourl])
+    @async_to_deferred
+    async def _get_refs(self, refs: list[str] | None = None) -> list[str]:
+        rows: str = await self._dovccmd(
+            'ls-remote', ['--refs', self.repourl] + (refs if refs is not None else [])
+        )
 
-        @d.addCallback
-        def parseRemote(rows):
-            branches = []
-            for row in rows.splitlines():
-                if '\t' not in row:
-                    # Not a useful line
-                    continue
-                sha, ref = row.split("\t")
-                branches.append(ref)
-            return branches
-        return d
+        branches: list[str] = []
+        for row in rows.splitlines():
+            if '\t' not in row:
+                # Not a useful line
+                continue
+            _, ref = row.split("\t")
+            branches.append(ref)
 
-    def _headsFilter(self, branch):
-        """Filter out remote references that don't begin with 'refs/heads'."""
-        return branch.startswith("refs/heads/")
+        return branches
+
+    @staticmethod
+    def _trim_prefix(value: str, prefix: str) -> str:
+        """Remove prefix from value."""
+        if value.startswith(prefix):
+            return value[len(prefix):]
+        return value
 
     def _removeHeads(self, branch):
         """Remove 'refs/heads/' prefix from remote references."""
@@ -178,10 +272,35 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             branch = branch[11:]
         return branch
 
-    def _trackerBranch(self, branch):
-        # manually quote tilde for Python 3.7
-        url = urlquote(self.repourl, '').replace('~', '%7E')
-        return "refs/buildbot/{}/{}".format(url, self._removeHeads(branch))
+    @staticmethod
+    def _tracker_ref(repourl: str, ref: str) -> str:
+        def _sanitize(value: str) -> str:
+            return urlquote(value, '').replace('~', '%7E')
+
+        tracker_prefix = "refs/buildbot"
+        # if ref is not a Git ref, store under a different path to avoid collision
+        if not ref.startswith('refs/'):
+            tracker_prefix += "/raw"
+
+        git_url = giturlparse(repourl)
+        if git_url is None:
+            # fallback to using the whole repourl
+            url_identifier = _sanitize(repourl)
+        else:
+            url_identifier = f"{git_url.proto}/{_sanitize(git_url.domain)}"
+            if git_url.port is not None:
+                url_identifier += f":{git_url.port}"
+
+            if git_url.owner is not None:
+                url_identifier += f"/{_sanitize(git_url.owner)}"
+            url_identifier += f"/{_sanitize(git_url.repo)}"
+
+        return f"{tracker_prefix}/{url_identifier}/{GitPoller._trim_prefix(ref, 'refs/')}"
+
+    def poll_should_exit(self):
+        # A single gitpoller loop may take a while on a loaded master, which would block
+        # reconfiguration, so we try to exit early.
+        return not self.doPoll.running
 
     @defer.inlineCallbacks
     def poll(self):
@@ -193,42 +312,58 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             log.msg(e.args[0])
             return
 
-        branches = self.branches if self.branches else []
-        remote_refs = yield self._getBranches()
-        if branches is True or callable(branches):
-            if callable(self.branches):
-                branches = [b for b in remote_refs if self.branches(b)]
-            else:
-                branches = [b for b in remote_refs if self._headsFilter(b)]
-        elif branches and remote_refs:
-            remote_branches = [self._removeHeads(b) for b in remote_refs]
-            branches = sorted(list(set(branches) & set(remote_branches)))
+        refs: list[str] = []
+        trim_ref_head = False
+        if callable(self.branches):
+            # Get all refs and let callback filter them
+            remote_refs = yield self._get_refs()
+            refs = [b for b in remote_refs if self.branches(b)]
+        elif self.branches is True:
+            # Get all branch refs
+            refs = yield self._get_refs(["refs/heads/*"])
+        elif self.branches:
+            refs = yield self._get_refs([f"refs/heads/{b}" for b in self.branches])
+            trim_ref_head = True
 
-        refspecs = [
-            '+{}:{}'.format(self._removeHeads(branch), self._trackerBranch(branch))
-            for branch in branches
-        ]
+        # Nothing to fetch and process.
+        if not refs:
+            return
+
+        if self.poll_should_exit():
+            return
+
+        refspecs = [f'+{ref}:{self._tracker_ref(self.repourl, ref)}' for ref in refs]
 
         try:
-            yield self._dovccmd('fetch', [self.repourl] + refspecs,
-                                path=self.workdir)
+            yield self._dovccmd(
+                'fetch', ['--progress', self.repourl] + refspecs + ['--'], path=self.workdir
+            )
         except GitError as e:
             log.msg(e.args[0])
             return
 
-        revs = {}
-        log.msg('gitpoller: processing changes from "{}"'.format(self.repourl))
-        for branch in branches:
-            try:
-                rev = yield self._dovccmd(
-                    'rev-parse', [self._trackerBranch(branch)], path=self.workdir)
-                revs[branch] = bytes2unicode(rev, self.encoding)
-                yield self._process_changes(revs[branch], branch)
-            except Exception:
-                log.err(_why="trying to poll branch {} of {}".format(
-                        branch, self.repourl))
+        if self.lastRev is None:
+            self.lastRev = yield self.getState('lastRev', {})
 
-        self.lastRev.update(revs)
+        revs = {}
+        log.msg(f'gitpoller: processing changes from "{self.repourl}"')
+        for ref in refs:
+            branch = ref if not trim_ref_head else self._trim_prefix(ref, 'refs/heads/')
+            try:
+                if self.poll_should_exit():  # pragma: no cover
+                    # Note that we still want to update the last known revisions for the branches
+                    # we did process
+                    break
+
+                rev = yield self._dovccmd(
+                    'rev-parse', [self._tracker_ref(self.repourl, ref)], path=self.workdir
+                )
+                revs[branch] = rev
+                yield self._process_changes(rev, branch)
+            except Exception:
+                log.err(_why=f"trying to poll branch {branch} of {self.repourl}")
+
+        self.lastRev = revs
         yield self.setState('lastRev', self.lastRev)
 
     def _get_commit_comments(self, rev):
@@ -248,10 +383,13 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
                     stamp = int(git_output)
                 except Exception as e:
                     log.msg(
-                        'gitpoller: caught exception converting output \'{}\' to timestamp'.format(git_output))
+                        f'gitpoller: caught exception converting output \'{git_output}\' to '
+                        'timestamp'
+                    )
                     raise e
                 return stamp
             return None
+
         return d
 
     def _get_commit_files(self, rev):
@@ -262,16 +400,18 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             # git use octal char sequences in quotes when non ASCII
             match = re.match('^"(.*)"$', file)
             if match:
-                file = bytes2unicode(match.groups()[0], encoding=self.encoding,
-                                     errors='unicode_escape')
+                file = bytes2unicode(
+                    match.groups()[0], encoding=self.encoding, errors='unicode_escape'
+                )
             return bytes2unicode(file, encoding=self.encoding)
 
         @d.addCallback
         def process(git_output):
-            fileList = [decode_file(file)
-                        for file in
-                        [s for s in git_output.splitlines() if len(s)]]
+            fileList = [
+                decode_file(file) for file in [s for s in git_output.splitlines() if len(s)]
+            ]
             return fileList
+
         return d
 
     def _get_commit_author(self, rev):
@@ -283,6 +423,7 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             if not git_output:
                 raise EnvironmentError('could not get commit author for rev')
             return git_output
+
         return d
 
     @defer.inlineCallbacks
@@ -308,10 +449,12 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             return
 
         # get the change list
-        revListArgs = (['--format=%H', '{}'.format(newRev)] +
-                       ['^' + rev
-                        for rev in sorted(self.lastRev.values())] +
-                       ['--'])
+        revListArgs = (
+            ['--ignore-missing']
+            + ['--format=%H', f'{newRev}']
+            + ['^' + rev for rev in sorted(self.lastRev.values())]
+            + ['--']
+        )
         self.changeCount = 0
         results = yield self._dovccmd('log', revListArgs, path=self.workdir)
 
@@ -325,29 +468,32 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
                 revList = [newRev]
                 if existingRev is None:
                     # This branch was completely unknown, rebuild
-                    log.msg('gitpoller: rebuilding {} for new branch "{}"'.format(
-                        newRev, branch))
+                    log.msg(f'gitpoller: rebuilding {newRev} for new branch "{branch}"')
                 else:
                     # This branch is known, but it now points to a different
                     # commit than last time we saw it, rebuild.
-                    log.msg('gitpoller: rebuilding {} for updated branch "{}"'.format(
-                        newRev, branch))
+                    log.msg(f'gitpoller: rebuilding {newRev} for updated branch "{branch}"')
 
         self.changeCount = len(revList)
         self.lastRev[branch] = newRev
 
         if self.changeCount:
-            log.msg('gitpoller: processing {} changes: {} from "{}" branch "{}"'.format(
-                    self.changeCount, revList, self.repourl, branch))
+            log.msg(
+                f'gitpoller: processing {self.changeCount} changes: {revList} from '
+                f'"{self.repourl}" branch "{branch}"'
+            )
 
         for rev in revList:
-            dl = defer.DeferredList([
-                self._get_commit_timestamp(rev),
-                self._get_commit_author(rev),
-                self._get_commit_committer(rev),
-                self._get_commit_files(rev),
-                self._get_commit_comments(rev),
-            ], consumeErrors=True)
+            dl = defer.DeferredList(
+                [
+                    self._get_commit_timestamp(rev),
+                    self._get_commit_author(rev),
+                    self._get_commit_committer(rev),
+                    self._get_commit_files(rev),
+                    self._get_commit_comments(rev),
+                ],
+                consumeErrors=True,
+            )
 
             results = yield dl
 
@@ -355,8 +501,7 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
             failures = [r[1] for r in results if not r[0]]
             if failures:
                 for failure in failures:
-                    log.err(
-                        failure, "while processing changes for {} {}".format(newRev, branch))
+                    log.err(failure, f"while processing changes for {rev} {branch}")
                 # just fail on the first error; they're probably all related!
                 failures[0].raiseException()
 
@@ -366,11 +511,15 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
                 author=author,
                 committer=committer,
                 revision=bytes2unicode(rev, encoding=self.encoding),
-                files=files, comments=comments, when_timestamp=timestamp,
+                files=files,
+                comments=comments,
+                when_timestamp=timestamp,
                 branch=bytes2unicode(self._removeHeads(branch)),
                 project=self.project,
                 repository=bytes2unicode(self.repourl, encoding=self.encoding),
-                category=self.category, src='git')
+                category=self.category,
+                src='git',
+            )
 
     def _isSshPrivateKeyNeededForCommand(self, command):
         commandsThatNeedKey = [
@@ -385,7 +534,7 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         # We change the permissions of the key file to be user-readable only so
         # that ssh does not complain. This is not used for security because the
         # parent directory will have proper permissions.
-        writeLocalFile(keyPath, self.sshPrivateKey, mode=stat.S_IRUSR)
+        writeLocalFile(keyPath, ensureSshKeyNewline(self.sshPrivateKey), mode=stat.S_IRUSR)
 
     def _downloadSshKnownHosts(self, path):
         if self.sshKnownHosts is not None:
@@ -404,7 +553,8 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
     def _dovccmd(self, command, args, path=None):
         if self._isSshPrivateKeyNeededForCommand(command):
             with private_tempdir.PrivateTemporaryDirectory(
-                    dir=self.workdir, prefix='.buildbot-ssh') as tmp_path:
+                dir=self.workdir, prefix='.buildbot-ssh'
+            ) as tmp_path:
                 stdout = yield self._dovccmdImpl(command, args, path, tmp_path)
         else:
             stdout = yield self._dovccmdImpl(command, args, path, None)
@@ -424,21 +574,26 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
                 known_hosts_path = self._getSshKnownHostsPath(ssh_workdir)
                 self._downloadSshKnownHosts(known_hosts_path)
 
-            self.adjustCommandParamsForSshPrivateKey(full_args, full_env,
-                                                     key_path, None,
-                                                     known_hosts_path)
+            self.adjustCommandParamsForSshPrivateKey(
+                full_args, full_env, key_path, None, known_hosts_path
+            )
 
         full_args += [command] + args
 
-        res = yield utils.getProcessOutputAndValue(self.gitbin,
-            full_args, path=path, env=full_env)
-        (stdout, stderr, code) = res
+        res = yield runprocess.run_process(
+            self.master.reactor, [self.gitbin] + full_args, path, env=full_env
+        )
+        (code, stdout, stderr) = res
         stdout = bytes2unicode(stdout, self.encoding)
         stderr = bytes2unicode(stderr, self.encoding)
         if code != 0:
             if code == 128:
-                raise GitError('command {} in {} on repourl {} failed with exit code {}: {}'.format(
-                               full_args, path, self.repourl, code, stderr))
-            raise EnvironmentError('command {} in {} on repourl {} failed with exit code {}: {}'.format(
-                                   full_args, path, self.repourl, code, stderr))
+                raise GitError(
+                    f'command {full_args} in {path} on repourl {self.repourl} failed '
+                    f'with exit code {code}: {stderr}'
+                )
+            raise EnvironmentError(
+                f'command {full_args} in {path} on repourl {self.repourl} '
+                f'failed with exit code {code}: {stderr}'
+            )
         return stdout.strip()
